==> m4e_sdk-0.0.1.tar.gz: test/services/m4e_auth_test.dart <==
import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/core/mocks/placeholder.dart' as placeholder;
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:m4e_sdk/src/models/shared/phone_number_model.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:mockito/mockito.dart';
import 'package:uuid_type/uuid_type.dart';
import 'package:matcher/matcher.dart';

class MockAuthApi extends Mock implements AuthApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

main() {
  M4eAuth _m4eAuth;

  MockAuthApi _mockAuthApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockAuthApi = MockAuthApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4eAuth = M4eAuth(
      authApi: _mockAuthApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _uniqueId =
      UniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');

  final _phoneNumber = PhoneNumber(country: 'GHANA', number: '+233708788110');

  final _authToken = AuthToken(id: _uniqueId, token: placeholder.token);

  final _m4eAuthPhoneNumberForm =
      M4eAuthPhoneNumberForm(id: _uniqueId, phoneNumber: _phoneNumber);

  final _m4eAuthOTPVerificationForm = M4eAuthOTPVerificationForm(
      id: _uniqueId, otpCode: '111111', phoneNumber: _phoneNumber);

  group('m4e-auth-test refreshAccessToken', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);

      // act
      await _m4eAuth.refreshAccessToken();

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should get existing token from cache', () async {
        // arrange
        when(_mockAuthApi.getAuthTokenFromCache())
            .thenAnswer((_) async => _authToken);
        when(_mockAuthApi.refreshAccessToken(any))
            .thenAnswer((_) async => _authToken);

        // act
        await _m4eAuth.refreshAccessToken();

        // assert
        verify(_mockAuthApi.getAuthTokenFromCache());
      });

      test('should return [AuthToken] if call to remote server is successful',
          () async {
        // arrange
        when(_mockAuthApi.getAuthTokenFromCache())
            .thenAnswer((_) async => _authToken);
        when(_mockAuthApi.refreshAccessToken(any))
            .thenAnswer((_) async => _authToken);

        // act
        final _result = await _m4eAuth.refreshAccessToken();

        // assert
        expect(_result, equals(_authToken));
        verify(_mockAuthApi.refreshAccessToken(_authToken));
        verify(_mockAuthApi.getAuthTokenFromCache());
      });

      test(
          'should call auth-api to cache returned [AuthToken] when call to remote server is successful',
          () async {
        // arrange
        when(_mockAuthApi.getAuthTokenFromCache())
            .thenAnswer((_) async => _authToken);
        when(_mockAuthApi.refreshAccessToken(any))
            .thenAnswer((_) async => _authToken);

        // act
        final _result = await _m4eAuth.refreshAccessToken();

        // assert
        expect(_result, equals(_authToken));
        verify(_mockAuthApi.refreshAccessToken(_authToken));
        verify(_mockAuthApi.getAuthTokenFromCache());
        verify(_mockAuthApi.cacheAuthToken(_authToken));
      });

      test('should throw a ServerException when server call fails', () async {
        // arrange
        when(_mockAuthApi.refreshAccessToken(any))
            .thenThrow(ExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eAuth.refreshAccessToken;

        // assert
        expect(() => _call(), throwsA(TypeMatcher<ServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockAuthApi.refreshAccessToken(any))
            .thenThrow(ExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eAuth.refreshAccessToken;

        // assert
        expect(() => _call(), throwsA(TypeMatcher<ServerException>()));
      });
    });
  });

  group('m4e-auth-test verifyPhoneNumber', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);

      // act
      await _m4eAuth.verifyPhoneNumber(_m4eAuthPhoneNumberForm);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return UniqueId on success call', () async {
        // arrange
        when(_mockAuthApi.verifyPhoneNumber(any))
            .thenAnswer((_) async => _uniqueId);

        // act
        final _result =
            await _m4eAuth.verifyPhoneNumber(_m4eAuthPhoneNumberForm);

        // assert
        expect(_result, equals(_uniqueId));
      });

      test('should throw a ServerException when server call fails', () async {
        // arrange
        when(_mockAuthApi.verifyPhoneNumber(any))
            .thenThrow(ExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eAuth.verifyPhoneNumber;

        // assert
        expect(() => _call(_m4eAuthPhoneNumberForm),
            throwsA(TypeMatcher<ServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockAuthApi.verifyPhoneNumber(any))
            .thenThrow(ExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eAuth.verifyPhoneNumber;

        // assert
        expect(() => _call(_m4eAuthPhoneNumberForm),
            throwsA(TypeMatcher<ServerException>()));
      });
    });
  });

  group('m4e-auth-test verifyOTPCode', () {
    final _tAuthToken = AuthToken(id: _uniqueId, token: placeholder.token);

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return AuthToken on success call', () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenAnswer((_) async => _tAuthToken);

        // act
        final _result =
            await _m4eAuth.verifyOTPCode(_m4eAuthOTPVerificationForm);

        // assert
        expect(_result, equals(_tAuthToken));
      });

      test('should cache AuthToken on success call', () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenAnswer((_) async => _tAuthToken);

        // act
        final _result =
            await _m4eAuth.verifyOTPCode(_m4eAuthOTPVerificationForm);

        // assert
        expect(_result, equals(_tAuthToken));
        verify(_mockAuthApi.cacheAuthToken(_tAuthToken));
      });

      test('should throw a ServerException when server call fails', () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenThrow(ExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eAuth.verifyOTPCode;

        // assert
        expect(() => _call(_m4eAuthOTPVerificationForm),
            throwsA(TypeMatcher<ServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenThrow(ExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eAuth.verifyOTPCode;

        // assert
        expect(() => _call(_m4eAuthOTPVerificationForm),
            throwsA(TypeMatcher<ServerException>()));
      });
    });
  });
}

==> m4e_sdk-0.0.3-dev.2.tar.gz: test/services/m4e_auth_test.dart <==
import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/src/core/core.dart';
import 'package:m4e_sdk/src/core/mocks/placeholder.dart' as placeholder;
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:m4e_sdk/src/m4e_forms/m4e_forms.dart';
import 'package:m4e_sdk/src/models/models.dart';
import 'package:m4e_sdk/src/models/shared/phone_number_model.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';

class MockAuthApi extends Mock implements AuthApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

main() {
  M4eAuthService _m4eAuth;

  MockAuthApi _mockAuthApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockAuthApi = MockAuthApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4eAuth = M4eAuthService(
      authApi: _mockAuthApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _uniqueId =
      M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0');

  final _phoneNumber =
      M4ePhoneNumber(country: 'GHANA', number: '+233708788110');

  final _authToken = M4eAuthToken(id: _uniqueId, token: placeholder.token);

  final _m4eAuthPhoneNumberForm =
      M4eAuthPhoneNumberForm(id: _uniqueId, phoneNumber: _phoneNumber);

  final _m4eAuthOTPVerificationForm = M4eAuthOTPVerificationForm(
      id: _uniqueId, otpCode: '111111', phoneNumber: _phoneNumber);

  group('m4e-auth-test refreshAccessToken', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);

      // act
      await _m4eAuth.refreshAccessToken();

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should get existing token from cache', () async {
        // arrange
        when(_mockAuthApi.getAuthTokenFromCache())
            .thenAnswer((_) async => _authToken);
        when(_mockAuthApi.refreshAccessToken(any))
            .thenAnswer((_) async => _authToken);

        // act
        await _m4eAuth.refreshAccessToken();

        // assert
        verify(_mockAuthApi.getAuthTokenFromCache());
      });

      test('should return [AuthToken] if call to remote server is successful',
          () async {
        // arrange
        when(_mockAuthApi.getAuthTokenFromCache())
            .thenAnswer((_) async => _authToken);
        when(_mockAuthApi.refreshAccessToken(any))
            .thenAnswer((_) async => _authToken);

        // act
        final _result = await _m4eAuth.refreshAccessToken();

        // assert
        expect(_result, equals(_authToken));
        verify(_mockAuthApi.refreshAccessToken(_authToken));
        verify(_mockAuthApi.getAuthTokenFromCache());
      });

      test(
          'should call auth-api to cache returned [AuthToken] when call to remote server is successful',
          () async {
        // arrange
        when(_mockAuthApi.getAuthTokenFromCache())
            .thenAnswer((_) async => _authToken);
        when(_mockAuthApi.refreshAccessToken(any))
            .thenAnswer((_) async => _authToken);

        // act
        final _result = await _m4eAuth.refreshAccessToken();

        // assert
        expect(_result, equals(_authToken));
        verify(_mockAuthApi.refreshAccessToken(_authToken));
        verify(_mockAuthApi.getAuthTokenFromCache());
        verify(_mockAuthApi.cacheAuthToken(_authToken));
      });

      test('should throw a ServerException when server call fails', () async {
        // arrange
        when(_mockAuthApi.refreshAccessToken(any))
            .thenThrow(M4eExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eAuth.refreshAccessToken;

        // assert
        expect(() => _call(), throwsA(TypeMatcher<M4eServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockAuthApi.refreshAccessToken(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eAuth.refreshAccessToken;

        // assert
        expect(() => _call(), throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });

  group('m4e-auth-test verifyPhoneNumber', () {
    test('should verify device connectivity status', () async {
      // arrange
      when(_mockDataConnectionChecker.hasConnection)
          .thenAnswer((_) async => true);

      // act
      await _m4eAuth.verifyPhoneNumber(_m4eAuthPhoneNumberForm);

      // assert
      verify(_mockDataConnectionChecker.hasConnection);
    });

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return UniqueId on success call', () async {
        // arrange
        when(_mockAuthApi.verifyPhoneNumber(any))
            .thenAnswer((_) async => _uniqueId);

        // act
        final _result =
            await _m4eAuth.verifyPhoneNumber(_m4eAuthPhoneNumberForm);

        // assert
        expect(_result, equals(_uniqueId));
      });

      test('should throw a ServerException when server call fails', () async {
        // arrange
        when(_mockAuthApi.verifyPhoneNumber(any))
            .thenThrow(M4eExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eAuth.verifyPhoneNumber;

        // assert
        expect(() => _call(_m4eAuthPhoneNumberForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockAuthApi.verifyPhoneNumber(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eAuth.verifyPhoneNumber;

        // assert
        expect(() => _call(_m4eAuthPhoneNumberForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });

  group('m4e-auth-test verifyOTPCode', () {
    final _tAuthToken = M4eAuthToken(id: _uniqueId, token: placeholder.token);

    group('isConnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);
      });

      test('should return AuthToken on success call', () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenAnswer((_) async => _tAuthToken);

        // act
        final _result =
            await _m4eAuth.verifyOTPCode(_m4eAuthOTPVerificationForm);

        // assert
        expect(_result, equals(_tAuthToken));
      });

      test('should cache AuthToken on success call', () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenAnswer((_) async => _tAuthToken);

        // act
        final _result =
            await _m4eAuth.verifyOTPCode(_m4eAuthOTPVerificationForm);

        // assert
        expect(_result, equals(_tAuthToken));
        verify(_mockAuthApi.cacheAuthToken(_tAuthToken));
      });

      test('should throw a ServerException when server call fails', () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenThrow(M4eExceptionMessages.kUndefinedException);

        // act
        final _call = _m4eAuth.verifyOTPCode;

        // assert
        expect(() => _call(_m4eAuthOTPVerificationForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });

    group('isDisconnected', () {
      setUp(() {
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => false);
      });

      test(
          'should throw a ServerException when no internet connection is detected',
          () async {
        // arrange
        when(_mockAuthApi.verifyOTPCode(any))
            .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

        // act
        final _call = _m4eAuth.verifyOTPCode;

        // assert
        expect(() => _call(_m4eAuthOTPVerificationForm),
            throwsA(TypeMatcher<M4eServerException>()));
      });
    });
  });
}
